package java.awt;

import java.awt.dnd.DropTarget;
import sun.security.action.GetBooleanAction;
import sun.awt.AWTAccessor;
import sun.java2d.pipe.Region;
import javax.accessibility.AccessibleComponent;
import javax.accessibility.AccessibleContext;
import javax.accessibility.Accessible;
import java.io.InvalidObjectException;
import java.io.OptionalDataException;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.beans.PropertyChangeListener;
import java.awt.event.KeyEvent;
import java.io.PrintWriter;
import java.io.PrintStream;
import sun.awt.SunToolkit;
import sun.awt.PeerEvent;
import javax.swing.JInternalFrame;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.EventListener;
import java.util.HashSet;
import java.util.Iterator;
import java.awt.peer.LightweightPeer;
import java.awt.event.ContainerEvent;
import java.awt.peer.ComponentPeer;
import java.awt.peer.ContainerPeer;
import java.util.ArrayList;
import sun.awt.AppContext;
import java.io.ObjectStreamField;
import java.awt.event.ContainerListener;
import java.util.Set;
import java.util.List;
import sun.util.logging.PlatformLogger;

public class Container extends Component
{
    private static final PlatformLogger log;
    private static final PlatformLogger eventLog;
    private static final Component[] EMPTY_ARRAY;
    private List<Component> component;
    LayoutManager layoutMgr;
    private LightweightDispatcher dispatcher;
    private transient FocusTraversalPolicy focusTraversalPolicy;
    private boolean focusCycleRoot;
    private boolean focusTraversalPolicyProvider;
    private transient Set<Thread> printingThreads;
    private transient boolean printing;
    transient ContainerListener containerListener;
    transient int listeningChildren;
    transient int listeningBoundsChildren;
    transient int descendantsCount;
    transient Color preserveBackgroundColor;
    private static final long serialVersionUID = 4613797578919906343L;
    static final boolean INCLUDE_SELF = true;
    static final boolean SEARCH_HEAVYWEIGHTS = true;
    private transient int numOfHWComponents;
    private transient int numOfLWComponents;
    private static final PlatformLogger mixingLog;
    private static final ObjectStreamField[] serialPersistentFields;
    private static final boolean isJavaAwtSmartInvalidate;
    private static boolean descendUnconditionallyWhenValidating;
    transient Component modalComp;
    transient AppContext modalAppContext;
    private int containerSerializedDataVersion;
    
    private static native void initIDs();
    
    public Container() {
        this.component = new ArrayList();
        this.focusCycleRoot = false;
        this.printing = false;
        this.preserveBackgroundColor = null;
        this.numOfHWComponents = 0;
        this.numOfLWComponents = 0;
        this.containerSerializedDataVersion = 1;
    }
    
    @Override
    void initializeFocusTraversalKeys() {
        this.focusTraversalKeys = new Set[4];
    }
    
    public int getComponentCount() {
        return this.countComponents();
    }
    
    @Deprecated
    public int countComponents() {
        return this.component.size();
    }
    
    public Component getComponent(int i) {
        try {
            return (Component)this.component.get(i);
        }
        catch (IndexOutOfBoundsException ex) {
            throw new ArrayIndexOutOfBoundsException("No such child: " + i);
        }
    }
    
    public Component[] getComponents() {
        return this.getComponents_NoClientCode();
    }
    
    final Component[] getComponents_NoClientCode() {
        return (Component[])this.component.toArray(Container.EMPTY_ARRAY);
    }
    
    Component[] getComponentsSync() {
        synchronized (this.getTreeLock()) {
            return this.getComponents();
        }
    }
    
    public Insets getInsets() {
        return this.insets();
    }
    
    @Deprecated
    public Insets insets() {
        ComponentPeer peer = this.peer;
        if ((peer instanceof ContainerPeer)) {
            return (Insets)((ContainerPeer)peer).getInsets().clone();
        }
        return new Insets(0, 0, 0, 0);
    }
    
    public Component add(Component component) {
        this.addImpl(component, null, -1);
        return component;
    }
    
    public Component add(String s, Component component) {
        this.addImpl(component, s, -1);
        return component;
    }
    
    public Component add(Component component, int n) {
        this.addImpl(component, null, n);
        return component;
    }
    
    private void checkAddToSelf(Component component) {
        if ((component instanceof Container)) {
            while (this != null) {
                if (this == component) {
                    throw new IllegalArgumentException("adding container's parent to itself");
                }
                this = this.parent;
            }
        }
    }
    
    private void checkNotAWindow(Component component) {
        if ((component instanceof Window)) {
            throw new IllegalArgumentException("adding a window to a container");
        }
        return;
    }
    
    private void checkAdding(Component component, int i) {
        this.checkTreeLock();
        GraphicsConfiguration graphicsConfiguration = this.getGraphicsConfiguration();
        if (i > this.component.size() || i < 0) {
            throw new IllegalArgumentException("illegal component position");
        }
        if (component.parent == this && i == this.component.size()) {
            throw new IllegalArgumentException("illegal component position " + i + " should be less then " + this.component.size());
        }
        this.checkAddToSelf(component);
        this.checkNotAWindow(component);
        if (this.getContainingWindow() != component.getContainingWindow()) {
            throw new IllegalArgumentException("component and container should be in the same top-level window");
        }
        if (graphicsConfiguration != null) {
            component.checkGD(graphicsConfiguration.getDevice().getIDstring());
        }
        return;
    }
    
    private boolean removeDelicately(Component child, Container container, int n) {
        this.checkTreeLock();
        int componentZOrder = this.getComponentZOrder(child);
        int removeNotifyNeeded = isRemoveNotifyNeeded(child, this, container);
        if (!(removeNotifyNeeded == 0)) {
            child.removeNotify();
        }
        if (container != this) {
            if (this.layoutMgr != null) {
                this.layoutMgr.removeLayoutComponent(child);
            }
            this.adjustListeningChildren(32768L, -child.numListening(32768L));
            this.adjustListeningChildren(65536L, -child.numListening(65536L));
            this.adjustDescendants(-child.countHierarchyMembers());
            child.parent = null;
            if (!(removeNotifyNeeded == 0)) {
                child.setGraphicsConfiguration(null);
            }
            this.component.remove(componentZOrder);
            this.invalidateIfValid();
        }
        else {
            this.component.remove(componentZOrder);
            this.component.add(n, child);
        }
        if (child.parent == null) {
            if (this.containerListener != null || (this.eventMask & 0x2L) != 0x0L || Toolkit.enabledOnToolkit(2L)) {
                this.dispatchEvent(new ContainerEvent(this, 301, child));
            }
            child.createHierarchyEvents(1400, child, this, 1L, Toolkit.enabledOnToolkit(32768L));
            if (this.peer != null && this.layoutMgr == null && this.isVisible()) {
                this.updateCursorImmediately();
            }
        }
        return removeNotifyNeeded;
    }
    
    boolean canContainFocusOwner(Component component) {
        if (!this.isEnabled() || !this.isDisplayable() || !this.isVisible() || !this.isFocusable()) {
            return false;
        }
        if (this.isFocusCycleRoot()) {
            FocusTraversalPolicy focusTraversalPolicy = this.getFocusTraversalPolicy();
            if (focusTraversalPolicy instanceof DefaultFocusTraversalPolicy && !((DefaultFocusTraversalPolicy)focusTraversalPolicy).accept(component)) {
                return false;
            }
        }
        synchronized (this.getTreeLock()) {
            if (this.parent != null) {
                return this.parent.canContainFocusOwner(component);
            }
        }
        return true;
    }
    
    final boolean hasHeavyweightDescendants() {
        this.checkTreeLock();
        return this.numOfHWComponents > 0;
    }
    
    final boolean hasLightweightDescendants() {
        this.checkTreeLock();
        return this.numOfLWComponents > 0;
    }
    
    Container getHeavyweightContainer() {
        this.checkTreeLock();
        if (this.peer != null && !(this.peer instanceof LightweightPeer)) {
            return this;
        }
        return this.getNativeContainer();
    }
    
    private static boolean isRemoveNotifyNeeded(Component component, Container container, Container container2) {
        if (container == null) {
            return false;
        }
        if (component.peer == null) {
            return false;
        }
        if (container2.peer == null) {
            return true;
        }
        if (component.isLightweight()) {
            int n = component instanceof Container;
            if (n == 0 || (n && !((Container)component).hasHeavyweightDescendants())) {
                return false;
            }
        }
        return container.getHeavyweightContainer() != container2.getHeavyweightContainer() && !component.peer.isReparentSupported();
    }
    
    public void setComponentZOrder(Component component, int n) {
        synchronized (this.getTreeLock()) {
            Container parent = component.parent;
            int componentZOrder = this.getComponentZOrder(component);
            if (parent == this && n == componentZOrder) {
                return;
            }
            this.checkAdding(component, n);
            int n2 = parent != null && parent.removeDelicately(component, this, n);
            this.addDelicately(component, parent, n);
            if (n2 == 0 && componentZOrder != -1) {
                component.mixOnZOrderChanging(componentZOrder, n);
            }
        }
    }
    
    private void reparentTraverse(ContainerPeer containerPeer, Container container) {
        this.checkTreeLock();
        for (int i = 0; i < container.getComponentCount(); ++i) {
            Component component = container.getComponent(i);
            if (component.isLightweight()) {
                if ((component instanceof Container)) {
                    this.reparentTraverse(containerPeer, (Container)component);
                }
            }
            else {
                component.getPeer().reparent(containerPeer);
            }
        }
    }
    
    private void reparentChild(Component component) {
        this.checkTreeLock();
        if (component == null) {
            return;
        }
        if (component.isLightweight()) {
            if ((component instanceof Container)) {
                this.reparentTraverse((ContainerPeer)this.getPeer(), (Container)component);
            }
        }
        else {
            component.getPeer().reparent((ContainerPeer)this.getPeer());
        }
        return;
    }
    
    private void addDelicately(Component child, Container container, int n) {
        this.checkTreeLock();
        if (container != this) {
            if (n == -1) {
                this.component.add(child);
            }
            else {
                this.component.add(n, child);
            }
            child.parent = this;
            child.setGraphicsConfiguration(this.getGraphicsConfiguration());
            this.adjustListeningChildren(32768L, child.numListening(32768L));
            this.adjustListeningChildren(65536L, child.numListening(65536L));
            this.adjustDescendants(child.countHierarchyMembers());
        }
        else if (n < this.component.size()) {
            this.component.set(n, child);
        }
        this.invalidateIfValid();
        if (this.peer != null) {
            if (child.peer == null) {
                child.addNotify();
            }
            else {
                Container heavyweightContainer = this.getHeavyweightContainer();
                if (container.getHeavyweightContainer() != heavyweightContainer) {
                    heavyweightContainer.reparentChild(child);
                }
                child.updateZOrder();
                if (!child.isLightweight() && this.isLightweight()) {
                    child.relocateComponent();
                }
            }
        }
        if (container != this) {
            if (this.layoutMgr != null) {
                if ((this.layoutMgr instanceof LayoutManager2)) {
                    ((LayoutManager2)this.layoutMgr).addLayoutComponent(child, null);
                }
                else {
                    this.layoutMgr.addLayoutComponent(null, child);
                }
            }
            if (this.containerListener != null || (this.eventMask & 0x2L) != 0x0L || Toolkit.enabledOnToolkit(2L)) {
                this.dispatchEvent(new ContainerEvent(this, 300, child));
            }
            child.createHierarchyEvents(1400, child, this, 1L, Toolkit.enabledOnToolkit(32768L));
            if (child.isFocusOwner() && !child.canBeFocusOwnerRecursively()) {
                child.transferFocus();
            }
            else if ((child instanceof Container)) {
                Component focusOwner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
                if (focusOwner != null && this.isParentOf(focusOwner) && !focusOwner.canBeFocusOwnerRecursively()) {
                    focusOwner.transferFocus();
                }
            }
        }
        else {
            child.createHierarchyEvents(1400, child, this, 1400L, Toolkit.enabledOnToolkit(32768L));
        }
        if (this.peer != null && this.layoutMgr == null && this.isVisible()) {
            this.updateCursorImmediately();
        }
    }
    
    public int getComponentZOrder(Component component) {
        if (component == null) {
            return -1;
        }
        synchronized (this.getTreeLock()) {
            if (component.parent != this) {
                return -1;
            }
            return this.component.indexOf(component);
        }
    }
    
    public void add(Component component, Object o) {
        this.addImpl(component, o, -1);
    }
    
    public void add(Component component, Object o, int n) {
        this.addImpl(component, o, n);
    }
    
    protected void addImpl(Component child, Object o, int n) {
        synchronized (this.getTreeLock()) {
            GraphicsConfiguration graphicsConfiguration = this.getGraphicsConfiguration();
            if (n > this.component.size() || (n < 0 && n != -1)) {
                throw new IllegalArgumentException("illegal component position");
            }
            this.checkAddToSelf(child);
            this.checkNotAWindow(child);
            if (child.parent != null) {
                child.parent.remove(child);
                if (n > this.component.size()) {
                    throw new IllegalArgumentException("illegal component position");
                }
            }
            if (graphicsConfiguration != null) {
                child.checkGD(graphicsConfiguration.getDevice().getIDstring());
            }
            if (n == -1) {
                this.component.add(child);
            }
            else {
                this.component.add(n, child);
            }
            child.parent = this;
            child.setGraphicsConfiguration(graphicsConfiguration);
            this.adjustListeningChildren(32768L, child.numListening(32768L));
            this.adjustListeningChildren(65536L, child.numListening(65536L));
            this.adjustDescendants(child.countHierarchyMembers());
            this.invalidateIfValid();
            if (this.peer != null) {
                child.addNotify();
            }
            if (this.layoutMgr != null) {
                if ((this.layoutMgr instanceof LayoutManager2)) {
                    ((LayoutManager2)this.layoutMgr).addLayoutComponent(child, o);
                }
                else if ((o instanceof String)) {
                    this.layoutMgr.addLayoutComponent((String)o, child);
                }
            }
            if (this.containerListener != null || (this.eventMask & 0x2L) != 0x0L || Toolkit.enabledOnToolkit(2L)) {
                this.dispatchEvent(new ContainerEvent(this, 300, child));
            }
            child.createHierarchyEvents(1400, child, this, 1L, Toolkit.enabledOnToolkit(32768L));
            if (this.peer != null && this.layoutMgr == null && this.isVisible()) {
                this.updateCursorImmediately();
            }
        }
    }
    
    @Override
    boolean updateGraphicsData(GraphicsConfiguration graphicsConfiguration) {
        this.checkTreeLock();
        int updateGraphicsData = super.updateGraphicsData(graphicsConfiguration);
        for (Component component : this.component) {
            if (component != null) {
                updateGraphicsData |= component.updateGraphicsData(graphicsConfiguration);
            }
            else {
                continue;
            }
        }
        return updateGraphicsData;
    }
    
    @Override
    void checkGD(String s) {
        for (Component component : this.component) {
            if (component != null) {
                component.checkGD(s);
            }
            else {
                continue;
            }
        }
    }
    
    public void remove(int index) {
        synchronized (this.getTreeLock()) {
            if (index < 0 || index >= this.component.size()) {
                throw new ArrayIndexOutOfBoundsException(index);
            }
            Component child = (Component)this.component.get(index);
            if (this.peer != null) {
                child.removeNotify();
            }
            if (this.layoutMgr != null) {
                this.layoutMgr.removeLayoutComponent(child);
            }
            this.adjustListeningChildren(32768L, -child.numListening(32768L));
            this.adjustListeningChildren(65536L, -child.numListening(65536L));
            this.adjustDescendants(-child.countHierarchyMembers());
            child.parent = null;
            this.component.remove(index);
            child.setGraphicsConfiguration(null);
            this.invalidateIfValid();
            if (this.containerListener != null || (this.eventMask & 0x2L) != 0x0L || Toolkit.enabledOnToolkit(2L)) {
                this.dispatchEvent(new ContainerEvent(this, 301, child));
            }
            child.createHierarchyEvents(1400, child, this, 1L, Toolkit.enabledOnToolkit(32768L));
            if (this.peer != null && this.layoutMgr == null && this.isVisible()) {
                this.updateCursorImmediately();
            }
        }
    }
    
    public void remove(Component component) {
        synchronized (this.getTreeLock()) {
            if (component.parent == this) {
                int index = this.component.indexOf(component);
                if (index >= 0) {
                    this.remove(index);
                }
            }
        }
    }
    
    public void removeAll() {
        synchronized (this.getTreeLock()) {
            this.adjustListeningChildren(32768L, -this.listeningChildren);
            this.adjustListeningChildren(65536L, -this.listeningBoundsChildren);
            this.adjustDescendants(-this.descendantsCount);
            while (!this.component.isEmpty()) {
                Component child = (Component)this.component.remove(this.component.size() - 1);
                if (this.peer != null) {
                    child.removeNotify();
                }
                if (this.layoutMgr != null) {
                    this.layoutMgr.removeLayoutComponent(child);
                }
                child.parent = null;
                child.setGraphicsConfiguration(null);
                if (this.containerListener != null || (this.eventMask & 0x2L) != 0x0L || Toolkit.enabledOnToolkit(2L)) {
                    this.dispatchEvent(new ContainerEvent(this, 301, child));
                }
                child.createHierarchyEvents(1400, child, this, 1L, Toolkit.enabledOnToolkit(32768L));
            }
            if (this.peer != null && this.layoutMgr == null && this.isVisible()) {
                this.updateCursorImmediately();
            }
            this.invalidateIfValid();
        }
    }
    
    @Override
    int numListening(long n) {
        int numListening = super.numListening(n);
        if (n == 32768L) {
            if (Container.eventLog.isLoggable(PlatformLogger.Level.FINE)) {
                int n3 = 0;
                Iterator<Component> iterator2 = this.component.iterator();
                while (iterator2.hasNext()) {
                    n3 += ((Component)iterator2.next()).numListening(n);
                }
                if (this.listeningChildren != n3) {
                    Container.eventLog.fine("Assertion (listeningChildren == sum) failed");
                }
            }
            return this.listeningChildren + numListening;
        }
        else if (n == 65536L) {
            if (Container.eventLog.isLoggable(PlatformLogger.Level.FINE)) {
                int n2 = 0;
                Iterator<Component> iterator = this.component.iterator();
                while (iterator.hasNext()) {
                    n2 += ((Component)iterator.next()).numListening(n);
                }
                if (this.listeningBoundsChildren != n2) {
                    Container.eventLog.fine("Assertion (listeningBoundsChildren == sum) failed");
                }
            }
            return this.listeningBoundsChildren + numListening;
        }
        else {
            if (Container.eventLog.isLoggable(PlatformLogger.Level.FINE)) {
                Container.eventLog.fine("This code must never be reached");
            }
            return numListening;
        }
    }
    
    void adjustListeningChildren(long n, int n2) {
        if (Container.eventLog.isLoggable(PlatformLogger.Level.FINE) && n != 32768L && n != 65536L && n != 98304L) {
            Container.eventLog.fine("Assertion failed");
        }
        if (n2 == 0) {
            return;
        }
        if ((n & 0x8000L) != 0x0L) {
            this.listeningChildren += n2;
        }
        if ((n & 0x10000L) != 0x0L) {
            this.listeningBoundsChildren += n2;
        }
        this.adjustListeningChildrenOnParent(n, n2);
        return;
    }
    
    void adjustDescendants(int n) {
        if (n == 0) {
            return;
        }
        this.descendantsCount += n;
        this.adjustDecendantsOnParent(n);
        return;
    }
    
    void adjustDecendantsOnParent(int n) {
        if (this.parent != null) {
            this.parent.adjustDescendants(n);
        }
    }
    
    @Override
    int countHierarchyMembers() {
        if (Container.log.isLoggable(PlatformLogger.Level.FINE)) {
            int n = 0;
            Iterator<Component> iterator = this.component.iterator();
            while (iterator.hasNext()) {
                n += ((Component)iterator.next()).countHierarchyMembers();
            }
            if (this.descendantsCount != n) {
                Container.log.fine("Assertion (descendantsCount == sum) failed");
            }
        }
        return this.descendantsCount + 1;
    }
    
    private int getListenersCount(int n, boolean b) {
        this.checkTreeLock();
        if (b) {
            return this.descendantsCount;
        }
        switch (n) {
            case 1400: {
                return this.listeningChildren;
            }
            case 1401:
            case 1402: {
                return this.listeningBoundsChildren;
            }
            default: {
                return 0;
            }
        }
    }
    
    @Override
    final int createHierarchyEvents(int n, Component component, Container container, long n2, boolean b) {
        this.checkTreeLock();
        int i = this.getListenersCount(n, b);
        for (int n3 = 0; i > 0; i -= ((Component)this.component.get(n3)).createHierarchyEvents(n, component, container, n2, b), ++n3) {}
        return i + super.createHierarchyEvents(n, component, container, n2, b);
    }
    
    final void createChildHierarchyEvents(int n, long n2, boolean b) {
        this.checkTreeLock();
        if (this.component.isEmpty()) {
            return;
        }
        for (int i = this.getListenersCount(n, b), n3 = 0; i > 0; i -= ((Component)this.component.get(n3)).createHierarchyEvents(n, this, this.parent, n2, b), ++n3) {}
        return;
    }
    
    public LayoutManager getLayout() {
        return this.layoutMgr;
    }
    
    public void setLayout(LayoutManager layoutMgr) {
        this.layoutMgr = layoutMgr;
        this.invalidateIfValid();
    }
    
    @Override
    public void doLayout() {
        this.layout();
    }
    
    @Deprecated
    @Override
    public void layout() {
        LayoutManager layoutMgr = this.layoutMgr;
        if (layoutMgr != null) {
            layoutMgr.layoutContainer(this);
        }
    }
    
    public boolean isValidateRoot() {
        return false;
    }
    
    @Override
    void invalidateParent() {
        if (!Container.isJavaAwtSmartInvalidate || !this.isValidateRoot()) {
            super.invalidateParent();
        }
    }
    
    @Override
    public void invalidate() {
        LayoutManager layoutMgr = this.layoutMgr;
        if ((layoutMgr instanceof LayoutManager2)) {
            ((LayoutManager2)layoutMgr).invalidateLayout(this);
        }
        super.invalidate();
    }
    
    @Override
    public void validate() {
        boolean visible = false;
        synchronized (this.getTreeLock()) {
            if ((!this.isValid() || Container.descendUnconditionallyWhenValidating) && this.peer != null) {
                ContainerPeer containerPeer = null;
                if ((this.peer instanceof ContainerPeer)) {
                    containerPeer = (ContainerPeer)this.peer;
                }
                if (containerPeer != null) {
                    containerPeer.beginValidate();
                }
                this.validateTree();
                if (containerPeer != null) {
                    containerPeer.endValidate();
                    if (!Container.descendUnconditionallyWhenValidating) {
                        visible = this.isVisible();
                    }
                }
            }
        }
        if (!(visible == 0)) {
            this.updateCursorImmediately();
        }
    }
    
    final void validateUnconditionally() {
        boolean visible = false;
        synchronized (this.getTreeLock()) {
            Container.descendUnconditionallyWhenValidating = true;
            this.validate();
            if ((this.peer instanceof ContainerPeer)) {
                visible = this.isVisible();
            }
            Container.descendUnconditionallyWhenValidating = false;
        }
        if (!(visible == 0)) {
            this.updateCursorImmediately();
        }
    }
    
    protected void validateTree() {
        this.checkTreeLock();
        if (!this.isValid() || Container.descendUnconditionallyWhenValidating) {
            if ((this.peer instanceof ContainerPeer)) {
                ((ContainerPeer)this.peer).beginLayout();
            }
            if (!this.isValid()) {
                this.doLayout();
            }
            for (int i = 0; i < this.component.size(); ++i) {
                Component component = (Component)this.component.get(i);
                if (component instanceof Container && !(component instanceof Window) && (!component.isValid() || Container.descendUnconditionallyWhenValidating)) {
                    ((Container)component).validateTree();
                }
                else {
                    component.validate();
                }
            }
            if ((this.peer instanceof ContainerPeer)) {
                ((ContainerPeer)this.peer).endLayout();
            }
        }
        super.validate();
    }
    
    void invalidateTree() {
        synchronized (this.getTreeLock()) {
            for (int i = 0; i < this.component.size(); ++i) {
                Component component = (Component)this.component.get(i);
                if ((component instanceof Container)) {
                    ((Container)component).invalidateTree();
                }
                else {
                    component.invalidateIfValid();
                }
            }
            this.invalidateIfValid();
        }
    }
    
    @Override
    public void setFont(Font font) {
        Font font2 = this.getFont();
        super.setFont(font);
        Font font3 = this.getFont();
        if (font3 != font2 && (font2 == null || !font2.equals(font3))) {
            this.invalidateTree();
        }
    }
    
    @Override
    public Dimension getPreferredSize() {
        return this.preferredSize();
    }
    
    @Deprecated
    @Override
    public Dimension preferredSize() {
        Dimension dimension = this.prefSize;
        if (dimension == null || (!this.isPreferredSizeSet() && !this.isValid())) {
            synchronized (this.getTreeLock()) {
                this.prefSize = ((this.layoutMgr != null) ? this.layoutMgr.preferredLayoutSize(this) : super.preferredSize());
                dimension = this.prefSize;
            }
        }
        if (dimension != null) {
            return new Dimension(dimension);
        }
        return dimension;
    }
    
    @Override
    public Dimension getMinimumSize() {
        return this.minimumSize();
    }
    
    @Deprecated
    @Override
    public Dimension minimumSize() {
        Dimension dimension = this.minSize;
        if (dimension == null || (!this.isMinimumSizeSet() && !this.isValid())) {
            synchronized (this.getTreeLock()) {
                this.minSize = ((this.layoutMgr != null) ? this.layoutMgr.minimumLayoutSize(this) : super.minimumSize());
                dimension = this.minSize;
            }
        }
        if (dimension != null) {
            return new Dimension(dimension);
        }
        return dimension;
    }
    
    @Override
    public Dimension getMaximumSize() {
        Dimension dimension = this.maxSize;
        if (dimension == null || (!this.isMaximumSizeSet() && !this.isValid())) {
            synchronized (this.getTreeLock()) {
                if ((this.layoutMgr instanceof LayoutManager2)) {
                    this.maxSize = ((LayoutManager2)this.layoutMgr).maximumLayoutSize(this);
                }
                else {
                    this.maxSize = super.getMaximumSize();
                }
                dimension = this.maxSize;
            }
        }
        if (dimension != null) {
            return new Dimension(dimension);
        }
        return dimension;
    }
    
    @Override
    public float getAlignmentX() {
        float n;
        if ((this.layoutMgr instanceof LayoutManager2)) {
            synchronized (this.getTreeLock()) {
                n = ((LayoutManager2)this.layoutMgr).getLayoutAlignmentX(this);
            }
        }
        else {
            n = super.getAlignmentX();
        }
        return n;
    }
    
    @Override
    public float getAlignmentY() {
        float n;
        if ((this.layoutMgr instanceof LayoutManager2)) {
            synchronized (this.getTreeLock()) {
                n = ((LayoutManager2)this.layoutMgr).getLayoutAlignmentY(this);
            }
        }
        else {
            n = super.getAlignmentY();
        }
        return n;
    }
    
    @Override
    public void paint(Graphics g) {
        if (this.isShowing()) {
            synchronized (this.getObjectLock()) {
                if (this.printing && this.printingThreads.contains(Thread.currentThread())) {
                    return;
                }
            }
            GraphicsCallback.PaintCallback.getInstance().runComponents(this.getComponentsSync(), g, 2);
        }
    }
    
    @Override
    public void update(Graphics graphics) {
        if (this.isShowing()) {
            if (!(this.peer instanceof LightweightPeer)) {
                graphics.clearRect(0, 0, this.width, this.height);
            }
            this.paint(graphics);
        }
    }
    
    @Override
    public void print(Graphics g) {
        if (this.isShowing()) {
            Thread currentThread = Thread.currentThread();
            try {
                synchronized (this.getObjectLock()) {
                    if (this.printingThreads == null) {
                        this.printingThreads = new HashSet();
                    }
                    this.printingThreads.add(currentThread);
                    this.printing = true;
                }
                super.print(g);
            }
            finally {
                synchronized (this.getObjectLock()) {
                    this.printingThreads.remove(currentThread);
                    this.printing = !this.printingThreads.isEmpty();
                }
            }
            GraphicsCallback.PrintCallback.getInstance().runComponents(this.getComponentsSync(), g, 2);
        }
    }
    
    public void paintComponents(Graphics g) {
        if (this.isShowing()) {
            GraphicsCallback.PaintAllCallback.getInstance().runComponents(this.getComponentsSync(), g, 4);
        }
    }
    
    @Override
    void lightweightPaint(Graphics graphics) {
        super.lightweightPaint(graphics);
        this.paintHeavyweightComponents(graphics);
    }
    
    @Override
    void paintHeavyweightComponents(Graphics g) {
        if (this.isShowing()) {
            GraphicsCallback.PaintHeavyweightComponentsCallback.getInstance().runComponents(this.getComponentsSync(), g, 3);
        }
    }
    
    public void printComponents(Graphics g) {
        if (this.isShowing()) {
            GraphicsCallback.PrintAllCallback.getInstance().runComponents(this.getComponentsSync(), g, 4);
        }
    }
    
    @Override
    void lightweightPrint(Graphics graphics) {
        super.lightweightPrint(graphics);
        this.printHeavyweightComponents(graphics);
    }
    
    @Override
    void printHeavyweightComponents(Graphics g) {
        if (this.isShowing()) {
            GraphicsCallback.PrintHeavyweightComponentsCallback.getInstance().runComponents(this.getComponentsSync(), g, 3);
        }
    }
    
    public synchronized void addContainerListener(ContainerListener containerListener) {
        if (containerListener == null) {
            return;
        }
        this.containerListener = AWTEventMulticaster.add(this.containerListener, containerListener);
        this.newEventsOnly = true;
        return;
    }
    
    public synchronized void removeContainerListener(ContainerListener containerListener) {
        if (containerListener == null) {
            return;
        }
        this.containerListener = AWTEventMulticaster.remove(this.containerListener, containerListener);
        return;
    }
    
    public synchronized ContainerListener[] getContainerListeners() {
        return (ContainerListener[])this.getListeners(ContainerListener.class);
    }
    
    @Override
    public <T extends EventListener> T[] getListeners(Class<T> clazz) {
        if (clazz == ContainerListener.class) {
            return AWTEventMulticaster.getListeners(this.containerListener, clazz);
        }
        return super.getListeners(clazz);
    }
    
    @Override
    boolean eventEnabled(AWTEvent awtEvent) {
        int id = awtEvent.getID();
        if (id == 300 || id == 301) {
            return (this.eventMask & 0x2L) != 0x0L || this.containerListener != null;
        }
        return super.eventEnabled(awtEvent);
    }
    
    @Override
    protected void processEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof ContainerEvent)) {
            this.processContainerEvent((ContainerEvent)awtEvent);
            return;
        }
        super.processEvent(awtEvent);
        return;
    }
    
    protected void processContainerEvent(ContainerEvent containerEvent) {
        ContainerListener containerListener = this.containerListener;
        if (containerListener != null) {
            switch (containerEvent.getID()) {
                case 300: {
                    containerListener.componentAdded(containerEvent);
                    break;
                }
                case 301: {
                    containerListener.componentRemoved(containerEvent);
                    break;
                }
            }
        }
    }
    
    @Override
    void dispatchEventImpl(AWTEvent awtEvent) {
        if (this.dispatcher != null && this.dispatcher.dispatchEvent(awtEvent)) {
            awtEvent.consume();
            if (this.peer != null) {
                this.peer.handleEvent(awtEvent);
            }
            return;
        }
        else {
            super.dispatchEventImpl(awtEvent);
            synchronized (this.getTreeLock()) {
                switch (awtEvent.getID()) {
                    case 101: {
                        this.createChildHierarchyEvents(1402, 0L, Toolkit.enabledOnToolkit(65536L));
                        break;
                    }
                    case 100: {
                        this.createChildHierarchyEvents(1401, 0L, Toolkit.enabledOnToolkit(65536L));
                        break;
                    }
                }
            }
            return;
        }
    }
    
    void dispatchEventToSelf(AWTEvent awtEvent) {
        super.dispatchEventImpl(awtEvent);
    }
    
    Component getMouseEventTarget(int n, int n2, boolean b) {
        return this.getMouseEventTarget(n, n2, b, Container.MouseEventTargetFilter.FILTER, false);
    }
    
    Component getDropTargetEventTarget(int n, int n2, boolean b) {
        return this.getMouseEventTarget(n, n2, b, Container.DropTargetEventTargetFilter.FILTER, true);
    }
    
    private Component getMouseEventTarget(int n, int n2, boolean b, Container.EventTargetFilter eventTargetFilter, boolean b2) {
        Component component = null;
        if (b2) {
            component = this.getMouseEventTargetImpl(n, n2, b, eventTargetFilter, true, b2);
        }
        if (component == null || component == this) {
            component = this.getMouseEventTargetImpl(n, n2, b, eventTargetFilter, false, b2);
            return component;
        }
        return component;
    }
    
    private Component getMouseEventTargetImpl(int n, int n2, boolean b, Container.EventTargetFilter eventTargetFilter, boolean b2, boolean b3) {
        synchronized (this.getTreeLock()) {
            for (int i = 0; i < this.component.size(); ++i) {
                Component component = (Component)this.component.get(i);
                if (component != null && component.visible && ((!b2 && component.peer instanceof LightweightPeer) || (b2 && !(component.peer instanceof LightweightPeer))) && component.contains(n - component.x, n2 - component.y)) {
                    if ((component instanceof Container)) {
                        Container container = (Container)component;
                        Component mouseEventTarget = container.getMouseEventTarget(n - container.x, n2 - container.y, b, eventTargetFilter, b3);
                        if (mouseEventTarget != null) {
                            return mouseEventTarget;
                        }
                    }
                    else if (eventTargetFilter.accept(component)) {
                        return component;
                    }
                }
            }
            int n3 = this.peer instanceof LightweightPeer || b;
            if (this.contains(n, n2) && n3 && eventTargetFilter.accept(this)) {
                return this;
            }
            return null;
        }
    }
    
    void proxyEnableEvents(long n) {
        if ((this.peer instanceof LightweightPeer)) {
            if (this.parent != null) {
                this.parent.proxyEnableEvents(n);
            }
        }
        else if (this.dispatcher != null) {
            this.dispatcher.enableEvents(n);
        }
    }
    
    @Deprecated
    @Override
    public void deliverEvent(Event event) {
        Component component = this.getComponentAt(event.x, event.y);
        if (component != null && component != this) {
            event.translate(-component.x, -component.y);
            component.deliverEvent(event);
        }
        else {
            this.postEvent(event);
        }
    }
    
    @Override
    public Component getComponentAt(int n, int n2) {
        return this.locate(n, n2);
    }
    
    @Deprecated
    @Override
    public Component locate(int n, int n2) {
        if (!this.contains(n, n2)) {
            return null;
        }
        Component component = null;
        synchronized (this.getTreeLock()) {
            for (Component component2 : this.component) {
                if (component2.contains(n - component2.x, n2 - component2.y)) {
                    if (!component2.isLightweight()) {
                        return component2;
                    }
                    if (component == null) {
                        component = component2;
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
        }
        return (component != null) ? component : this;
    }
    
    @Override
    public Component getComponentAt(Point point) {
        return this.getComponentAt(point.x, point.y);
    }
    
    public Point getMousePosition(boolean b) throws HeadlessException {
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        PointerInfo pointerInfo = (PointerInfo)AccessController.doPrivileged(new PrivilegedAction<PointerInfo>() {
            @Override
            public PointerInfo run() {
                return MouseInfo.getPointerInfo();
            }
        });
        synchronized (this.getTreeLock()) {
            if (this.isSameOrAncestorOf(this.findUnderMouseInWindow(pointerInfo), b)) {
                return this.pointRelativeToComponent(pointerInfo.getLocation());
            }
            return null;
        }
    }
    
    @Override
    boolean isSameOrAncestorOf(Component component, boolean b) {
        return this == component || (b && this.isParentOf(component));
    }
    
    public Component findComponentAt(int n, int n2) {
        return this.findComponentAt(n, n2, true);
    }
    
    final Component findComponentAt(int n, int n2, boolean b) {
        synchronized (this.getTreeLock()) {
            if (this.isRecursivelyVisible()) {
                return this.findComponentAtImpl(n, n2, b);
            }
        }
        return null;
    }
    
    final Component findComponentAtImpl(int n, int n2, boolean b) {
        if (!this.contains(n, n2) || !this.visible || (!b && !this.enabled)) {
            return null;
        }
        Component child = null;
        for (Component component : this.component) {
            int n3 = n - component.x;
            int n4 = n2 - component.y;
            if (!component.contains(n3, n4)) {
                continue;
            }
            if (!component.isLightweight()) {
                Component child2 = getChildAt(component, n3, n4, b);
                if (child2 != null) {
                    return child2;
                }
                continue;
            }
            else if (child == null) {
                child = getChildAt(component, n3, n4, b);
            }
            else {
                continue;
            }
        }
        return (child != null) ? child : this;
    }
    
    private static Component getChildAt(Component component, int n, int n2, boolean b) {
        if ((component instanceof Container)) {
            component = ((Container)component).findComponentAtImpl(n, n2, b);
        }
        else {
            component = component.getComponentAt(n, n2);
        }
        if (component != null && component.visible && (b || component.enabled)) {
            return component;
        }
        return null;
    }
    
    public Component findComponentAt(Point point) {
        return this.findComponentAt(point.x, point.y);
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            super.addNotify();
            if (!(this.peer instanceof LightweightPeer)) {
                this.dispatcher = new LightweightDispatcher(this);
            }
            for (int i = 0; i < this.component.size(); ++i) {
                ((Component)this.component.get(i)).addNotify();
            }
        }
    }
    
    @Override
    public void removeNotify() {
        synchronized (this.getTreeLock()) {
            for (int i = this.component.size() - 1; i >= 0; --i) {
                Component component = (Component)this.component.get(i);
                if (component != null) {
                    component.setAutoFocusTransferOnDisposal(false);
                    component.removeNotify();
                    component.setAutoFocusTransferOnDisposal(true);
                }
            }
            if (this.containsFocus() && KeyboardFocusManager.isAutoFocusTransferEnabledFor(this) && !this.transferFocus(false)) {
                this.transferFocusBackward(true);
            }
            if (this.dispatcher != null) {
                this.dispatcher.dispose();
                this.dispatcher = null;
            }
            super.removeNotify();
        }
    }
    
    public boolean isAncestorOf(Component component) {
        Container container;
        if (component == null || (container = component.getParent()) == null) {
            return false;
        }
        while (container != null) {
            if (container == this) {
                return true;
            }
            container = container.getParent();
        }
        return false;
    }
    
    private void startLWModal() {
        this.modalAppContext = AppContext.getAppContext();
        long mostRecentKeyEventTime = Toolkit.getEventQueue().getMostRecentKeyEventTime();
        Component component = Component.isInstanceOf(this, "javax.swing.JInternalFrame") ? ((JInternalFrame)(JInternalFrame)this).getMostRecentFocusOwner() : null;
        if (component != null) {
            KeyboardFocusManager.getCurrentKeyboardFocusManager().enqueueKeyEvents(mostRecentKeyEventTime, component);
        }
        final Container heavyweightContainer;
        synchronized (this.getTreeLock()) {
            heavyweightContainer = this.getHeavyweightContainer();
            if (heavyweightContainer.modalComp != null) {
                this.modalComp = heavyweightContainer.modalComp;
                heavyweightContainer.modalComp = this;
                return;
            }
            heavyweightContainer.modalComp = this;
        }
        Container.3 runnable = new Runnable() {
            @Override
            public void run() {
                ((EventDispatchThread)Thread.currentThread()).pumpEventsForHierarchy(new Conditional() {
                    @Override
                    public boolean evaluate() {
                        return Container.this.windowClosingException == null && heavyweightContainer.modalComp != null;
                    }
                }, Container.this);
            }
        };
        if (EventQueue.isDispatchThread()) {
            SequencedEvent currentSequencedEvent = KeyboardFocusManager.getCurrentKeyboardFocusManager().getCurrentSequencedEvent();
            if (currentSequencedEvent != null) {
                currentSequencedEvent.dispose();
            }
            runnable.run();
        }
        else {
            synchronized (this.getTreeLock()) {
                Toolkit.getEventQueue().postEvent(new PeerEvent(this, runnable, 1L));
                while (this.windowClosingException == null && heavyweightContainer.modalComp != null) {
                    try {
                        this.getTreeLock().wait();
                    }
                    catch (InterruptedException ex) {
                        break;
                    }
                }
            }
        }
        if (this.windowClosingException != null) {
            this.windowClosingException.fillInStackTrace();
            throw this.windowClosingException;
        }
        if (component != null) {
            KeyboardFocusManager.getCurrentKeyboardFocusManager().dequeueKeyEvents(mostRecentKeyEventTime, component);
        }
        return;
    }
    
    private void stopLWModal() {
        synchronized (this.getTreeLock()) {
            if (this.modalAppContext != null) {
                Container heavyweightContainer = this.getHeavyweightContainer();
                if (heavyweightContainer != null) {
                    if (this.modalComp != null) {
                        heavyweightContainer.modalComp = this.modalComp;
                        this.modalComp = null;
                        return;
                    }
                    heavyweightContainer.modalComp = null;
                }
                SunToolkit.postEvent(this.modalAppContext, new PeerEvent(this, new Container.WakingRunnable(), 1L));
            }
            EventQueue.invokeLater(new Container.WakingRunnable());
            this.getTreeLock().notifyAll();
        }
    }
    
    @Override
    protected String paramString() {
        String str = super.paramString();
        LayoutManager layoutMgr = this.layoutMgr;
        if (layoutMgr != null) {
            str = str + ",layout=" + layoutMgr.getClass().getName();
            return str;
        }
        return str;
    }
    
    @Override
    public void list(PrintStream printStream, int n) {
        super.list(printStream, n);
        synchronized (this.getTreeLock()) {
            for (int i = 0; i < this.component.size(); ++i) {
                Component component = (Component)this.component.get(i);
                if (component != null) {
                    component.list(printStream, n + 1);
                }
            }
        }
    }
    
    @Override
    public void list(PrintWriter printWriter, int n) {
        super.list(printWriter, n);
        synchronized (this.getTreeLock()) {
            for (int i = 0; i < this.component.size(); ++i) {
                Component component = (Component)this.component.get(i);
                if (component != null) {
                    component.list(printWriter, n + 1);
                }
            }
        }
    }
    
    @Override
    public void setFocusTraversalKeys(int n, Set<? extends AWTKeyStroke> set) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("invalid focus traversal key identifier");
        }
        this.setFocusTraversalKeys_NoIDCheck(n, set);
        return;
    }
    
    @Override
    public Set<AWTKeyStroke> getFocusTraversalKeys(int n) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("invalid focus traversal key identifier");
        }
        return this.getFocusTraversalKeys_NoIDCheck(n);
    }
    
    @Override
    public boolean areFocusTraversalKeysSet(int n) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("invalid focus traversal key identifier");
        }
        return this.focusTraversalKeys != null && this.focusTraversalKeys[n] != null;
    }
    
    @Override
    public boolean isFocusCycleRoot(Container container) {
        return (this.isFocusCycleRoot() && container == this) || super.isFocusCycleRoot(container);
    }
    
    private Container findTraversalRoot() {
        Container currentFocusCycleRoot = KeyboardFocusManager.getCurrentKeyboardFocusManager().getCurrentFocusCycleRoot();
        if (currentFocusCycleRoot == this) {
        }
        else {
            this = this.getFocusCycleRootAncestor();
            if (this == null) {
            }
        }
        if (this != currentFocusCycleRoot) {
            KeyboardFocusManager.getCurrentKeyboardFocusManager().setGlobalCurrentFocusCycleRootPriv(this);
            return this;
        }
        return this;
    }
    
    @Override
    final boolean containsFocus() {
        return this.isParentOf(KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner());
    }
    
    private boolean isParentOf(Component parent) {
        synchronized (this.getTreeLock()) {
            while (parent != null && parent != this && !(parent instanceof Window)) {
                parent = parent.getParent();
            }
            return parent == this;
        }
    }
    
    @Override
    void clearMostRecentFocusOwnerOnHide() {
        int n = 0;
        Window containingWindow = null;
        synchronized (this.getTreeLock()) {
            containingWindow = this.getContainingWindow();
            if (containingWindow != null) {
                Component mostRecentFocusOwner = KeyboardFocusManager.getMostRecentFocusOwner(containingWindow);
                n = (mostRecentFocusOwner == this || this.isParentOf(mostRecentFocusOwner));
                synchronized (KeyboardFocusManager.class) {
                    Component temporaryLostComponent = containingWindow.getTemporaryLostComponent();
                    if (this.isParentOf(temporaryLostComponent) || temporaryLostComponent == this) {
                        containingWindow.setTemporaryLostComponent(null);
                    }
                }
            }
        }
        if (n != 0) {
            KeyboardFocusManager.setMostRecentFocusOwner(containingWindow, null);
        }
    }
    
    @Override
    void clearCurrentFocusCycleRootOnHide() {
        KeyboardFocusManager currentKeyboardFocusManager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
        Container currentFocusCycleRoot = currentKeyboardFocusManager.getCurrentFocusCycleRoot();
        if (currentFocusCycleRoot == this || this.isParentOf(currentFocusCycleRoot)) {
            currentKeyboardFocusManager.setGlobalCurrentFocusCycleRootPriv(null);
        }
    }
    
    @Override
    final Container getTraversalRoot() {
        if (this.isFocusCycleRoot()) {
            return this.findTraversalRoot();
        }
        return super.getTraversalRoot();
    }
    
    public void setFocusTraversalPolicy(FocusTraversalPolicy focusTraversalPolicy) {
        FocusTraversalPolicy focusTraversalPolicy2;
        synchronized (this) {
            focusTraversalPolicy2 = this.focusTraversalPolicy;
            this.focusTraversalPolicy = focusTraversalPolicy;
        }
        this.firePropertyChange("focusTraversalPolicy", focusTraversalPolicy2, focusTraversalPolicy);
    }
    
    public FocusTraversalPolicy getFocusTraversalPolicy() {
        if (!this.isFocusTraversalPolicyProvider() && !this.isFocusCycleRoot()) {
            return null;
        }
        FocusTraversalPolicy focusTraversalPolicy = this.focusTraversalPolicy;
        if (focusTraversalPolicy != null) {
            return focusTraversalPolicy;
        }
        Container focusCycleRootAncestor = this.getFocusCycleRootAncestor();
        if (focusCycleRootAncestor != null) {
            return focusCycleRootAncestor.getFocusTraversalPolicy();
        }
        return KeyboardFocusManager.getCurrentKeyboardFocusManager().getDefaultFocusTraversalPolicy();
    }
    
    public boolean isFocusTraversalPolicySet() {
        return this.focusTraversalPolicy != null;
    }
    
    public void setFocusCycleRoot(boolean focusCycleRoot) {
        int focusCycleRoot2;
        synchronized (this) {
            focusCycleRoot2 = this.focusCycleRoot;
            this.focusCycleRoot = focusCycleRoot;
        }
        this.firePropertyChange("focusCycleRoot", (boolean)focusCycleRoot2, focusCycleRoot);
    }
    
    public boolean isFocusCycleRoot() {
        return this.focusCycleRoot;
    }
    
    public final void setFocusTraversalPolicyProvider(boolean focusTraversalPolicyProvider) {
        int focusTraversalPolicyProvider2;
        synchronized (this) {
            focusTraversalPolicyProvider2 = this.focusTraversalPolicyProvider;
            this.focusTraversalPolicyProvider = focusTraversalPolicyProvider;
        }
        this.firePropertyChange("focusTraversalPolicyProvider", (boolean)focusTraversalPolicyProvider2, focusTraversalPolicyProvider);
    }
    
    public final boolean isFocusTraversalPolicyProvider() {
        return this.focusTraversalPolicyProvider;
    }
    
    public void transferFocusDownCycle() {
        if (this.isFocusCycleRoot()) {
            KeyboardFocusManager.getCurrentKeyboardFocusManager().setGlobalCurrentFocusCycleRootPriv(this);
            Component defaultComponent = this.getFocusTraversalPolicy().getDefaultComponent(this);
            if (defaultComponent != null) {
                defaultComponent.requestFocus(CausedFocusEvent.Cause.TRAVERSAL_DOWN);
            }
        }
    }
    
    void preProcessKeyEvent(KeyEvent keyEvent) {
        Container parent = this.parent;
        if (parent != null) {
            parent.preProcessKeyEvent(keyEvent);
        }
    }
    
    void postProcessKeyEvent(KeyEvent keyEvent) {
        Container parent = this.parent;
        if (parent != null) {
            parent.postProcessKeyEvent(keyEvent);
        }
    }
    
    @Override
    boolean postsOldMouseEvents() {
        return true;
    }
    
    @Override
    public void applyComponentOrientation(ComponentOrientation componentOrientation) {
        super.applyComponentOrientation(componentOrientation);
        synchronized (this.getTreeLock()) {
            for (int i = 0; i < this.component.size(); ++i) {
                ((Component)this.component.get(i)).applyComponentOrientation(componentOrientation);
            }
        }
    }
    
    @Override
    public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        super.addPropertyChangeListener(propertyChangeListener);
    }
    
    @Override
    public void addPropertyChangeListener(String s, PropertyChangeListener propertyChangeListener) {
        super.addPropertyChangeListener(s, propertyChangeListener);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("ncomponents", this.component.size());
        putFields.put("component", this.component.toArray(Container.EMPTY_ARRAY));
        putFields.put("layoutMgr", this.layoutMgr);
        putFields.put("dispatcher", this.dispatcher);
        putFields.put("maxSize", this.maxSize);
        putFields.put("focusCycleRoot", this.focusCycleRoot);
        putFields.put("containerSerializedDataVersion", this.containerSerializedDataVersion);
        putFields.put("focusTraversalPolicyProvider", this.focusTraversalPolicyProvider);
        objectOutputStream.writeFields();
        AWTEventMulticaster.save(objectOutputStream, "containerL", this.containerListener);
        objectOutputStream.writeObject(null);
        if ((this.focusTraversalPolicy instanceof Serializable)) {
            objectOutputStream.writeObject(this.focusTraversalPolicy);
        }
        else {
            objectOutputStream.writeObject(null);
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        Component[] empty_ARRAY = (Component[])(Component[])fields.get("component", null);
        if (empty_ARRAY == null) {
            empty_ARRAY = Container.EMPTY_ARRAY;
        }
        int intValue = (int)Integer.valueOf(fields.get("ncomponents", 0));
        if (intValue < 0 || intValue > empty_ARRAY.length) {
            throw new InvalidObjectException("Incorrect number of components");
        }
        this.component = new ArrayList(intValue);
        for (int i = 0; i < intValue; ++i) {
            this.component.add(empty_ARRAY[i]);
        }
        this.layoutMgr = (LayoutManager)fields.get("layoutMgr", null);
        this.dispatcher = (LightweightDispatcher)fields.get("dispatcher", null);
        if (this.maxSize == null) {
            this.maxSize = (Dimension)fields.get("maxSize", null);
        }
        this.focusCycleRoot = fields.get("focusCycleRoot", false);
        this.containerSerializedDataVersion = fields.get("containerSerializedDataVersion", 1);
        this.focusTraversalPolicyProvider = fields.get("focusTraversalPolicyProvider", false);
        for (Component component : this.component) {
            (component.parent = this).adjustListeningChildren(32768L, component.numListening(32768L));
            this.adjustListeningChildren(65536L, component.numListening(65536L));
            this.adjustDescendants(component.countHierarchyMembers());
        }
        Object object;
        while (null != (object = objectInputStream.readObject())) {
            if ("containerL" == ((String)object).intern()) {
                this.addContainerListener((ContainerListener)(ContainerListener)objectInputStream.readObject());
            }
            else {
                objectInputStream.readObject();
            }
        }
        try {
            Object object2 = objectInputStream.readObject();
            if ((object2 instanceof FocusTraversalPolicy)) {
                this.focusTraversalPolicy = (FocusTraversalPolicy)object2;
            }
        }
        catch (OptionalDataException ex) {
            if (!ex.eof) {
                throw ex;
            }
        }
        return;
    }
    
    Accessible getAccessibleAt(Point point) {
        synchronized (this.getTreeLock()) {
            if ((this instanceof Accessible)) {
                AccessibleContext accessibleContext = ((Accessible)this).getAccessibleContext();
                if (accessibleContext != null) {
                    for (int accessibleChildrenCount = accessibleContext.getAccessibleChildrenCount(), j = 0; j < accessibleChildrenCount; ++j) {
                        Accessible accessibleChild = accessibleContext.getAccessibleChild(j);
                        if (accessibleChild != null) {
                            accessibleContext = accessibleChild.getAccessibleContext();
                            if (accessibleContext != null) {
                                AccessibleComponent accessibleComponent = accessibleContext.getAccessibleComponent();
                                if (accessibleComponent != null && accessibleComponent.isShowing()) {
                                    Point location2 = accessibleComponent.getLocation();
                                    if (accessibleComponent.contains(new Point(point.x - location2.x, point.y - location2.y))) {
                                        return accessibleChild;
                                    }
                                }
                            }
                        }
                    }
                }
                return (Accessible)this;
            }
            else {
                if (!this.contains(point.x, point.y)) {
                    this = null;
                }
                else {
                    for (int componentCount = this.getComponentCount(), i = 0; i < componentCount; ++i) {
                        Component component = this.getComponent(i);
                        if (component != null && component.isShowing()) {
                            Point location = component.getLocation();
                            if (component.contains(point.x - location.x, point.y - location.y)) {
                                this = component;
                            }
                        }
                    }
                }
                if ((this instanceof Accessible)) {
                    return (Accessible)this;
                }
                return null;
            }
        }
    }
    
    int getAccessibleChildrenCount() {
        synchronized (this.getTreeLock()) {
            int n = 0;
            Component[] components = this.getComponents();
            for (int i = 0; i < components.length; ++i) {
                if ((components[i] instanceof Accessible)) {
                    ++n;
                }
            }
            return n;
        }
    }
    
    Accessible getAccessibleChild(int n) {
        synchronized (this.getTreeLock()) {
            Component[] components = this.getComponents();
            int n2 = 0;
            for (int i = 0; i < components.length; ++i) {
                if ((components[i] instanceof Accessible)) {
                    if (n2 == n) {
                        return (Accessible)components[i];
                    }
                    ++n2;
                }
            }
            return null;
        }
    }
    
    final void increaseComponentCount(Component component) {
        synchronized (this.getTreeLock()) {
            if (!component.isDisplayable()) {
                throw new IllegalStateException("Peer does not exist while invoking the increaseComponentCount() method");
            }
            int numOfHWComponents = 0;
            int numOfLWComponents = 0;
            if ((component instanceof Container)) {
                numOfLWComponents = ((Container)component).numOfLWComponents;
                numOfHWComponents = ((Container)component).numOfHWComponents;
            }
            if (component.isLightweight()) {
                ++numOfLWComponents;
            }
            else {
                ++numOfHWComponents;
            }
            while (this != null) {
                Container container = this;
                container.numOfLWComponents += numOfLWComponents;
                Container container2 = this;
                container2.numOfHWComponents += numOfHWComponents;
                this = this.getContainer();
            }
        }
    }
    
    final void decreaseComponentCount(Component component) {
        synchronized (this.getTreeLock()) {
            if (!component.isDisplayable()) {
                throw new IllegalStateException("Peer does not exist while invoking the decreaseComponentCount() method");
            }
            int numOfHWComponents = 0;
            int numOfLWComponents = 0;
            if ((component instanceof Container)) {
                numOfLWComponents = ((Container)component).numOfLWComponents;
                numOfHWComponents = ((Container)component).numOfHWComponents;
            }
            if (component.isLightweight()) {
                ++numOfLWComponents;
            }
            else {
                ++numOfHWComponents;
            }
            while (this != null) {
                Container container = this;
                container.numOfLWComponents -= numOfLWComponents;
                Container container2 = this;
                container2.numOfHWComponents -= numOfHWComponents;
                this = this.getContainer();
            }
        }
    }
    
    private int getTopmostComponentIndex() {
        this.checkTreeLock();
        if (this.getComponentCount() > 0) {
            return 0;
        }
        return -1;
    }
    
    private int getBottommostComponentIndex() {
        this.checkTreeLock();
        if (this.getComponentCount() > 0) {
            return this.getComponentCount() - 1;
        }
        return -1;
    }
    
    @Override
    final Region getOpaqueShape() {
        this.checkTreeLock();
        if (this.isLightweight() && this.isNonOpaqueForMixing() && this.hasLightweightDescendants()) {
            Region region = Region.EMPTY_REGION;
            for (int i = 0; i < this.getComponentCount(); ++i) {
                Component component = this.getComponent(i);
                if (component.isLightweight() && component.isShowing()) {
                    region = region.getUnion(component.getOpaqueShape());
                }
            }
            return region.getIntersection(this.getNormalShape());
        }
        return super.getOpaqueShape();
    }
    
    final void recursiveSubtractAndApplyShape(Region region) {
        this.recursiveSubtractAndApplyShape(region, this.getTopmostComponentIndex(), this.getBottommostComponentIndex());
    }
    
    final void recursiveSubtractAndApplyShape(Region region, int n) {
        this.recursiveSubtractAndApplyShape(region, n, this.getBottommostComponentIndex());
    }
    
    final void recursiveSubtractAndApplyShape(Region obj, int i, int j) {
        this.checkTreeLock();
        if (Container.mixingLog.isLoggable(PlatformLogger.Level.FINE)) {
            Container.mixingLog.fine("this = " + this + "; shape=" + obj + "; fromZ=" + i + "; toZ=" + j);
        }
        if (i == -1) {
            return;
        }
        if (obj.isEmpty()) {
            return;
        }
        if (this.getLayout() != null && !this.isValid()) {
            return;
        }
        Component component;
        for (; i <= j; ++i) {
            component = this.getComponent(i);
            if (!component.isLightweight()) {
                component.subtractAndApplyShape(obj);
            }
            else if (component instanceof Container && ((Container)component).hasHeavyweightDescendants() && component.isShowing()) {
                ((Container)component).recursiveSubtractAndApplyShape(obj);
            }
        }
        return;
    }
    
    final void recursiveApplyCurrentShape() {
        this.recursiveApplyCurrentShape(this.getTopmostComponentIndex(), this.getBottommostComponentIndex());
    }
    
    final void recursiveApplyCurrentShape(int n) {
        this.recursiveApplyCurrentShape(n, this.getBottommostComponentIndex());
    }
    
    final void recursiveApplyCurrentShape(int i, int j) {
        this.checkTreeLock();
        if (Container.mixingLog.isLoggable(PlatformLogger.Level.FINE)) {
            Container.mixingLog.fine("this = " + this + "; fromZ=" + i + "; toZ=" + j);
        }
        if (i == -1) {
            return;
        }
        if (this.getLayout() != null && !this.isValid()) {
            return;
        }
        Component component;
        for (; i <= j; ++i) {
            component = this.getComponent(i);
            if (!component.isLightweight()) {
                component.applyCurrentShape();
            }
            if (component instanceof Container && ((Container)component).hasHeavyweightDescendants()) {
                ((Container)component).recursiveApplyCurrentShape();
            }
        }
        return;
    }
    
    private void recursiveShowHeavyweightChildren() {
        if (!this.hasHeavyweightDescendants() || !this.isVisible()) {
            return;
        }
        for (int i = 0; i < this.getComponentCount(); ++i) {
            Component component = this.getComponent(i);
            if (component.isLightweight()) {
                if ((component instanceof Container)) {
                    ((Container)component).recursiveShowHeavyweightChildren();
                }
            }
            else if (component.isVisible()) {
                ComponentPeer peer = component.getPeer();
                if (peer != null) {
                    peer.setVisible(true);
                }
            }
        }
        return;
    }
    
    private void recursiveHideHeavyweightChildren() {
        if (!this.hasHeavyweightDescendants()) {
            return;
        }
        for (int i = 0; i < this.getComponentCount(); ++i) {
            Component component = this.getComponent(i);
            if (component.isLightweight()) {
                if ((component instanceof Container)) {
                    ((Container)component).recursiveHideHeavyweightChildren();
                }
            }
            else if (component.isVisible()) {
                ComponentPeer peer = component.getPeer();
                if (peer != null) {
                    peer.setVisible(false);
                }
            }
        }
        return;
    }
    
    private void recursiveRelocateHeavyweightChildren(Point point) {
        for (int i = 0; i < this.getComponentCount(); ++i) {
            Component component = this.getComponent(i);
            if (component.isLightweight()) {
                if (component instanceof Container && ((Container)component).hasHeavyweightDescendants()) {
                    Point o = new Point(point);
                    o.translate(component.getX(), component.getY());
                    ((Container)component).recursiveRelocateHeavyweightChildren(o);
                }
            }
            else {
                ComponentPeer peer = component.getPeer();
                if (peer != null) {
                    peer.setBounds(point.x + component.getX(), point.y + component.getY(), component.getWidth(), component.getHeight(), 1);
                }
            }
        }
    }
    
    final boolean isRecursivelyVisibleUpToHeavyweightContainer() {
        if (!this.isLightweight()) {
            return true;
        }
        while (this != null && this.isLightweight()) {
            if (!this.isVisible()) {
                return false;
            }
            this = this.getContainer();
        }
        return true;
    }
    
    @Override
    void mixOnShowing() {
        synchronized (this.getTreeLock()) {
            if (Container.mixingLog.isLoggable(PlatformLogger.Level.FINE)) {
                Container.mixingLog.fine("this = " + this);
            }
            int lightweight = this.isLightweight();
            if (lightweight && this.isRecursivelyVisibleUpToHeavyweightContainer()) {
                this.recursiveShowHeavyweightChildren();
            }
            if (!this.isMixingNeeded()) {
                return;
            }
            if (lightweight == 0 || (lightweight && this.hasHeavyweightDescendants())) {
                this.recursiveApplyCurrentShape();
            }
            super.mixOnShowing();
        }
    }
    
    @Override
    void mixOnHiding(boolean b) {
        synchronized (this.getTreeLock()) {
            if (Container.mixingLog.isLoggable(PlatformLogger.Level.FINE)) {
                Container.mixingLog.fine("this = " + this + "; isLightweight=" + b);
            }
            if (b) {
                this.recursiveHideHeavyweightChildren();
            }
            super.mixOnHiding(b);
        }
    }
    
    @Override
    void mixOnReshaping() {
        synchronized (this.getTreeLock()) {
            if (Container.mixingLog.isLoggable(PlatformLogger.Level.FINE)) {
                Container.mixingLog.fine("this = " + this);
            }
            int mixingNeeded = this.isMixingNeeded();
            if (this.isLightweight() && this.hasHeavyweightDescendants()) {
                Point point = new Point(this.getX(), this.getY());
                for (Container container = this.getContainer(); container != null && container.isLightweight(); container = container.getContainer()) {
                    point.translate(container.getX(), container.getY());
                }
                this.recursiveRelocateHeavyweightChildren(point);
                if (mixingNeeded == 0) {
                    return;
                }
                this.recursiveApplyCurrentShape();
            }
            if (mixingNeeded == 0) {
                return;
            }
            super.mixOnReshaping();
        }
    }
    
    @Override
    void mixOnZOrderChanging(int i, int j) {
        synchronized (this.getTreeLock()) {
            if (Container.mixingLog.isLoggable(PlatformLogger.Level.FINE)) {
                Container.mixingLog.fine("this = " + this + "; oldZ=" + i + "; newZ=" + j);
            }
            if (!this.isMixingNeeded()) {
                return;
            }
            if (j < i && this.isLightweight() && this.hasHeavyweightDescendants()) {
                this.recursiveApplyCurrentShape();
            }
            super.mixOnZOrderChanging(i, j);
        }
    }
    
    @Override
    void mixOnValidating() {
        synchronized (this.getTreeLock()) {
            if (Container.mixingLog.isLoggable(PlatformLogger.Level.FINE)) {
                Container.mixingLog.fine("this = " + this);
            }
            if (!this.isMixingNeeded()) {
                return;
            }
            if (this.hasHeavyweightDescendants()) {
                this.recursiveApplyCurrentShape();
            }
            if (this.isLightweight() && this.isNonOpaqueForMixing()) {
                this.subtractAndApplyShapeBelowMe();
            }
            super.mixOnValidating();
        }
    }
    
    static {
        log = PlatformLogger.getLogger("java.awt.Container");
        eventLog = PlatformLogger.getLogger("java.awt.event.Container");
        EMPTY_ARRAY = new Component[0];
        mixingLog = PlatformLogger.getLogger("java.awt.mixing.Container");
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("ncomponents", Integer.TYPE), new ObjectStreamField("component", Component[].class), new ObjectStreamField("layoutMgr", LayoutManager.class), new ObjectStreamField("dispatcher", LightweightDispatcher.class), new ObjectStreamField("maxSize", Dimension.class), new ObjectStreamField("focusCycleRoot", Boolean.TYPE), new ObjectStreamField("containerSerializedDataVersion", Integer.TYPE), new ObjectStreamField("focusTraversalPolicyProvider", Boolean.TYPE) };
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        AWTAccessor.setContainerAccessor(new AWTAccessor.ContainerAccessor() {
            @Override
            public void validateUnconditionally(Container container) {
                container.validateUnconditionally();
            }
            
            @Override
            public Component findComponentAt(Container container, int n, int n2, boolean b) {
                return container.findComponentAt(n, n2, b);
            }
        });
        isJavaAwtSmartInvalidate = (boolean)AccessController.doPrivileged(new GetBooleanAction("java.awt.smartInvalidate"));
        Container.descendUnconditionallyWhenValidating = false;
    }
    
    static class MouseEventTargetFilter implements Container.EventTargetFilter
    {
        static final Container.EventTargetFilter FILTER;
        
        private MouseEventTargetFilter() {
        }
        
        @Override
        public boolean accept(Component component) {
            return (component.eventMask & 0x20L) != 0x0L || (component.eventMask & 0x10L) != 0x0L || (component.eventMask & 0x20000L) != 0x0L || component.mouseListener != null || component.mouseMotionListener != null || component.mouseWheelListener != null;
        }
        
        static {
            FILTER = new Container.MouseEventTargetFilter();
        }
    }
    
    static class DropTargetEventTargetFilter implements Container.EventTargetFilter
    {
        static final Container.EventTargetFilter FILTER;
        
        private DropTargetEventTargetFilter() {
        }
        
        @Override
        public boolean accept(Component component) {
            DropTarget dropTarget = component.getDropTarget();
            return dropTarget != null && dropTarget.isActive();
        }
        
        static {
            FILTER = new Container.DropTargetEventTargetFilter();
        }
    }
    
    static final class WakingRunnable implements Runnable
    {
        @Override
        public void run() {
        }
    }
    
    protected class AccessibleAWTContainer extends Component.AccessibleAWTComponent
    {
        private static final long serialVersionUID = 5081320404842566097L;
        private transient volatile int propertyListenersCount;
        protected ContainerListener accessibleContainerHandler;
        
        protected AccessibleAWTContainer() {
            this.propertyListenersCount = 0;
            this.accessibleContainerHandler = null;
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return Container.this.getAccessibleChildrenCount();
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            return Container.this.getAccessibleChild(n);
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            return Container.this.getAccessibleAt(point);
        }
        
        @Override
        public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
            if (this.accessibleContainerHandler == null) {
                this.accessibleContainerHandler = new Container.AccessibleAWTContainer.AccessibleContainerHandler();
            }
            if (this.propertyListenersCount++ == 0) {
                Container.this.addContainerListener(this.accessibleContainerHandler);
            }
            super.addPropertyChangeListener(propertyChangeListener);
        }
        
        @Override
        public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
            int propertyListenersCount = this.propertyListenersCount - 1;
            this.propertyListenersCount = propertyListenersCount;
            if (propertyListenersCount == 0) {
                Container.this.removeContainerListener(this.accessibleContainerHandler);
            }
            super.removePropertyChangeListener(propertyChangeListener);
        }
        
        protected class AccessibleContainerHandler implements ContainerListener
        {
            @Override
            public void componentAdded(ContainerEvent containerEvent) {
                Component child = containerEvent.getChild();
                if (child != null && child instanceof Accessible) {
                    Container.AccessibleAWTContainer.this.firePropertyChange("AccessibleChild", null, ((Accessible)child).getAccessibleContext());
                }
            }
            
            @Override
            public void componentRemoved(ContainerEvent containerEvent) {
                Component child = containerEvent.getChild();
                if (child != null && child instanceof Accessible) {
                    Container.AccessibleAWTContainer.this.firePropertyChange("AccessibleChild", ((Accessible)child).getAccessibleContext(), null);
                }
            }
        }
    }
    
    interface EventTargetFilter
    {
        public abstract boolean accept(Component p0);
    }
}
